
package org.ululatus.sleepybot;

import java.util.*;
import java.io.*;

import sleep.interfaces.*;
import sleep.runtime.*;
import sleep.bridges.*;
import sleep.parser.ParserConfig;

import org.jibble.pircbot.DccChat;

public class DccChatBridge implements Loadable, Function
{
//	private static ScriptedBot scriptbot;
	private ScriptedBot scriptbot;
	
	static { // add Predicate keywords
		ParserConfig.addKeyword("-isdccchat");
	}
	
	public DccChatBridge( ScriptedBot bot )
	{
		scriptbot = bot;
	}
	
	public boolean scriptLoaded(ScriptInstance script)
	{
		Hashtable environment = script.getScriptEnvironment().getEnvironment();
		environment.put("&dccSendChatRequest", new DccSendChatRequest());
		environment.put("&chatReadLine", new DccChatReadLine());

		IsDccChat isDccChat = new IsDccChat();
		environment.put("-isdccchat", isDccChat );
		environment.put("&isDccChat", isDccChat );
		
		environment.put("&chatAccept", this );
		environment.put("&chatSendLine", this );
		environment.put("&chatClose", this );
		environment.put("&chatGetNick", this );
		environment.put("&chatGetLogin", this );
		environment.put("&chatGetHostname", this );
		environment.put("&chatGetBufferedReader", this );
		environment.put("&chatGetBufferedWriter", this );
		
		return true;
	}
	
	public boolean scriptUnloaded(ScriptInstance script)
	{
		return true;
	}
	
	public Scalar evaluate(String name, ScriptInstance script, Stack args)
	{
		Object obj = BridgeUtilities.getObject( args );
		if ( obj instanceof DccChat)
		{
			DccChat chat = (DccChat) obj;
			if ( name.equals("&chatAccept") )
			{ //public synchronized void accept() throws IOException {
				try
				{
					chat.accept();
				}
				catch ( IOException e )
				{
					script.getScriptEnvironment().flagError( e.toString() );
				}
			}
			else if ( name.equals("&chatSendLine") )
			{ // public void sendLine(String line) throws IOException {
				String line = BridgeUtilities.getString( args, "" );
				if ( !line.equals("") )
				try
				{
					chat.sendLine( line );
				}
				catch ( IOException e )
				{
					script.getScriptEnvironment().flagError( e.toString() );
				}
			}
			else if ( name.equals("&chatClose") )
			{ // public void close() throws IOException {
				try
				{
					chat.close();
				}
				catch ( IOException e )
				{
					script.getScriptEnvironment().flagError( e.toString() );
				}
			}
			else if ( name.equals("&chatGetNick") )
			{ // public String getNick() {
				return SleepUtils.getScalar( chat.getNick() );
			}
			else if ( name.equals("&chatGetLogin") )
			{ // public String getLogin() 
				return SleepUtils.getScalar( chat.getLogin() );
			}
			else if ( name.equals("&chatGetHostname") )
			{ // public String getHostname() {
				return SleepUtils.getScalar( chat.getLogin() );
			}
			/*else if ( name.equals("&chatGetBufferedReader") )
			{ // public BufferedReader getBufferedReader() {
			}
			else if ( name.equals("&chatGetBufferedWriter") )
			{ // public BufferedWriter getBufferedWriter() {
			}*/
		}
		return SleepUtils.getEmptyScalar();
	}

	private class IsDccChat implements Predicate, Function
	{
		public boolean decide(String name, ScriptInstance script, Stack args)
		{
			Object a = BridgeUtilities.getObject( args );
			return ( a instanceof DccChat );
		}
		
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			Object obj = BridgeUtilities.getObject( args );
			if ( obj instanceof DccChat )
				return SleepUtils.getScalar(1);
			return SleepUtils.getEmptyScalar();
		}
	}
	
	//public final DccChat dccSendChatRequest(String nick, int timeout) {
	private class DccSendChatRequest implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			String nick = BridgeUtilities.getString( args, "" );
			int timeout = BridgeUtilities.getInt( args, 120000 );
			/*Thread thread = new Thread( new Runnable() {
					public void run()
					{
						DccChat chat = UserBridge.this.scriptbot.dccSendChatRequest( nick, timeout );
						UserBridge.this.scriptbot.onChatRequestAccepted( chat );
					}
				}
			); */
			Thread thread = new Thread( new DccChatThread( nick, timeout ) );
			thread.setName( nick + "-" +System.currentTimeMillis() );
			thread.setDaemon(true); // we don't want the virtual machine staying alive.
      		thread.setPriority(Thread.MIN_PRIORITY);
			thread.start();
			return SleepUtils.getEmptyScalar();
		}
	}

	private class DccChatReadLine implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			Object obj = BridgeUtilities.getObject( args );
			if ( obj instanceof DccChat )
			{
				final DccChat chat = (DccChat) obj;
				
				/*Thread thread = new Thread( new Runnable() {
					public void run()
					{
						String line = chat.readLine();
						scriptbot.onChatReadLine( chat, line );
					}
				}
				); */
				Thread thread = new Thread( new DccChatThread( chat ) );
				thread.setName( chat.getNick() + "-" +System.currentTimeMillis() );
				thread.setDaemon(true); // we don't want the virtual machine staying alive.
	      		thread.setPriority(Thread.MIN_PRIORITY);
				thread.start();
			}
			return SleepUtils.getEmptyScalar();
		}
	}

	private final class DccChatThread implements Runnable
	{
		private DccChat chat;
		private String nick;
		private int timeout;
		DccChatThread( DccChat c )
		{
			chat = c;
		}
		
		DccChatThread( String n, int to )
		{
			nick = n;
			timeout = to;
		}
		
		public void run()
		{
			if ( chat == null )
			{
				DccChat chat = DccChatBridge.this.scriptbot.dccSendChatRequest( nick, timeout );
				DccChatBridge.this.scriptbot.onChatRequestAccepted( chat );
			}
			else
			{
				try
				{
					String line = chat.readLine();
					DccChatBridge.this.scriptbot.onChatReadLine( chat, line );
				}
				catch ( IOException e ) { /* DO NOTHING */ }
			}
		}
	}
}
