
package org.ululatus.sleepybot;

import java.util.*;
import java.io.*;

import sleep.interfaces.*;
import sleep.runtime.*;
import sleep.bridges.*;
import sleep.parser.ParserConfig;

import org.jibble.pircbot.DccFileTransfer;

public class DccFileTransferBridge implements Loadable, Function
{
//	private static ScriptedBot scriptbot;
	private ScriptedBot scriptbot;
	
	static { // add Predicate keywords
		ParserConfig.addKeyword("-isdccfiletransfer");
		ParserConfig.addKeyword("-isdft");
	}
	
	public DccFileTransferBridge( ScriptedBot bot )
	{
		scriptbot = bot;
	}
	
	public boolean scriptLoaded(ScriptInstance script)
	{
		Hashtable environment = script.getScriptEnvironment().getEnvironment();
		
		environment.put("&dccSendFile", new DccSendFile()); // MOVED TO ScriptedBotBridge
		
		IsDccFileTransfer idft = new IsDccFileTransfer();
		environment.put("-isdccfiletransfer", idft);
		environment.put("-isdft", idft);
		environment.put("&isDccFileTransfer", idft);
		
		environment.put("&dftReceive", this );
		environment.put("&dftGetNick", this );
		environment.put("&dftGetLogin", this );
		environment.put("&dftGetHostname", this );
		environment.put("&dftGetFile", this );
		environment.put("&dftGetPort", this );
		environment.put("&dftIsIncoming", this );
		environment.put("&dftIsOutgoing", this );
		environment.put("&dftSetPacketDelay", this );
		environment.put("&dftGetPacketDelay", this );
		environment.put("&dftGetSize", this );
		environment.put("&dftGetProgress", this );
		environment.put("&dftGetProgressPercentage", this );
		environment.put("&dftGetTransferRate", this );
		environment.put("&dftClose", this );
				
		return true;
	}
	
	public boolean scriptUnloaded(ScriptInstance script)
	{
		return true;
	}
	
	public Scalar evaluate(String name, ScriptInstance script, Stack args)
	{
		Object obj = BridgeUtilities.getObject( args );
		if ( obj instanceof DccFileTransfer)
		{
			if ( name.equals("&dftReceive") )
			{ // public synchronized void receive(File file, boolean resume) {
				Object fobj = BridgeUtilities.getObject( args );
				if ( fobj instanceof File )
				{
					boolean resume = false;
					if ( BridgeUtilities.getInt( args, 0 ) == 1 )
						resume = true;
					((DccFileTransfer) obj).receive( ((File) fobj), resume );
				}
			}
			else if ( name.equals("&dftGetNick") )
			{ // public String getNick() {
				return SleepUtils.getScalar( ((DccFileTransfer) obj).getNick() );
			}
			else if ( name.equals("&dftGetLogin") )
			{ // public String getLogin() {
				return SleepUtils.getScalar( ((DccFileTransfer) obj).getLogin() );
			}
			else if ( name.equals("&dftGetHostname") )
			{ // public String getHostname() {
				return SleepUtils.getScalar( ((DccFileTransfer) obj).getHostname() );
			}
			else if ( name.equals("&dftGetFile") )
			{ // public File getFile() {
				return SleepUtils.getScalar( ((DccFileTransfer) obj).getFile() );
			}
			else if ( name.equals("&dftGetPort") )
			{ // public int getPort() {
				return SleepUtils.getScalar( ((DccFileTransfer) obj).getPort() );
			}
			else if ( name.equals("&dftIsIncoming") )
			{ // public boolean isIncoming() {
				if ( ((DccFileTransfer) obj).isIncoming() )
					return SleepUtils.getScalar( 1 );
			}
			else if ( name.equals("&dftIsOutgoing") )
			{ // public boolean isOutgoing() {
				if ( ((DccFileTransfer) obj).isOutgoing() )
					return SleepUtils.getScalar( 1 );
			}
			else if ( name.equals("&dftSetPacketDelay") )
			{ // public void setPacketDelay(long millis) {
				long delay = BridgeUtilities.getLong( args, 0L );
				if ( delay >= 0L )
					((DccFileTransfer) obj).setPacketDelay( delay );
			}
			else if ( name.equals("&dftGetPacketDelay") )
			{ // public long getPacketDelay() {
				return SleepUtils.getScalar( ((DccFileTransfer) obj).getPacketDelay() );
			}
			else if ( name.equals("&dftGetSize") )
			{ // public long getSize() {
				return SleepUtils.getScalar( ((DccFileTransfer) obj).getSize() );
			}
			else if ( name.equals("&dftGetProgress") )
			{ // public long getProgress() {
				return SleepUtils.getScalar( ((DccFileTransfer) obj).getProgress() );
			}
			else if ( name.equals("&dftGetProgressPercentage") )
			{ // public double getProgressPercentage() {
				return SleepUtils.getScalar( ((DccFileTransfer) obj).getProgressPercentage() );
			}
			else if ( name.equals("&dftGetTransferRate") )
			{ // public long getTransferRate() {
				return SleepUtils.getScalar( ((DccFileTransfer) obj).getTransferRate() );
			}
			else if ( name.equals("&dftClose") )
			{ // public void close() {
				((DccFileTransfer) obj).close();
			}
		}
		return SleepUtils.getEmptyScalar();
	}
	
    //public final DccFileTransfer dccSendFile(File file, String nick, int timeout) {
//	MOVED BACK TO ScriptedBotBridge
	private class DccSendFile implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			//LinkedList errs = new LinkedList();
			Object obj = BridgeUtilities.getObject( args );
			File file = null;
			if ( !(obj instanceof File) )
			{
				script.getScriptEnvironment().flagError("dccSendFile: first parameter does not evaluate to a java.io.File");
				return SleepUtils.getEmptyScalar();
			}
			else file = (File) obj;
			
			String nick = BridgeUtilities.getString( args, "" );
			if ( nick.equals("") )
			{
				script.getScriptEnvironment().flagError("dccSendFile: second parameter(nick) not specified");
				return SleepUtils.getEmptyScalar();
			}
			
			int timeout = BridgeUtilities.getInt( args, 120000 );
			
			Scalar dccFileTransfer = SleepUtils.getScalar( DccFileTransferBridge.this.scriptbot.dccSendFile( file, nick, timeout ) );
			
			return dccFileTransfer;
		}
	}
	
	private class IsDccFileTransfer implements Predicate, Function
	{
		public boolean decide(String name, ScriptInstance script, Stack args)
		{
			Object a = BridgeUtilities.getObject( args );
			return ( a instanceof DccFileTransfer );
		}
		
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			Object obj = BridgeUtilities.getObject( args );
			if ( obj instanceof DccFileTransfer )
				return SleepUtils.getScalar(1);
			return SleepUtils.getEmptyScalar();
		}
	}

}