
package org.ululatus.sleepybot;

import org.jibble.pircbot.*;

import sleep.interfaces.*;
import sleep.runtime.*;
import sleep.error.*;

import java.io.*;
import java.util.*;

import java.lang.reflect.*;
import sleep.engine.*;
import sleep.bridges.*;

public class ScriptedBot extends PircBot implements RuntimeWarningWatcher
{
	public void processScriptWarning(ScriptWarning warning)    
	{
		StringBuffer buffer = new StringBuffer(warning.getNameShort());
		buffer.append("(line ").append(warning.getLineNumber()).append("): ");
		buffer.append(warning.getMessage());
		String output = buffer.toString();
		System.err.println( output );
		log( output );
	}
	
	//static Scalar arguments;
	private Scalar arguments;
		
	private ScriptLoader loader;
	private ScriptInstance script;
	private ScriptEnvironment env;
	private boolean loaded = false;

	private boolean verbose = false;
	private final boolean debug = Boolean.getBoolean("scriptedbot.debug");
	
	private Field code;
	private HashMap blocks = new HashMap();
	
	private final String _name_ = "SleepyBot";
	private final String _login_ = _name_;
	private final String _VERSION_ = "1.0";
	private final String _version_ = _name_ + " " + _VERSION_;
	
	public ScriptedBot( String scriptfile ) throws IOException
	{
		arguments = toSleepArray( new String[] { scriptfile.toString() } );
		init( scriptfile );
	}
	
	public ScriptedBot( String[] args ) throws IOException
	{
		arguments = toSleepArray( args );
		init( args[0] );
	}

	Scalar getArguments()
	{
		return arguments;
	}

	private void init( String scriptfile ) throws IOException
	{
		code = getCodeField();
		if ( code == null )
			throw new RuntimeException("Can't access function blocks");
		
		setName(_name_);
		setLogin(_login_);
		setVersion(_version_);
				
		loader = new ScriptLoader();
		loader.addGlobalBridge( new ScriptedBotBridge( this ) );
		loader.addGlobalBridge( new UserBridge( this ) );
		loader.addGlobalBridge( new DccChatBridge( this ) );
		loader.addGlobalBridge( new DccFileTransferBridge( this ) );
		loader.addGlobalBridge( new UtilitiesBridge() );
//		loader.addGlobalBridge( new IncludeBridge() ); // waiting for sleep 2.1 :)

		try
		{
			script = loader.loadScript( scriptfile );
			loaded = true;
		}
		catch ( YourCodeSucksException e )
		{
			Iterator i = e.getErrors().iterator();
			while (i.hasNext())
			{
				SyntaxError error = (SyntaxError)i.next();
				String description = error.getDescription();
				String code = error.getCodeSnippet();
				int lineNumber = error.getLineNumber();
				System.out.println( description + " at " + lineNumber + ": " + code );
			}
		}
		if ( loaded )
		{
			script.addWarningWatcher(this);
			script.runScript();
			env = loader.getFirstScriptEnvironment();
			
			Stack args = new Stack();
			HashMap params = new HashMap();
			if ( hasFunction("&init") ) {
				callFunction("&init", args, params );
			}
			else {
				throw new RuntimeException("missing init function");
			}
		}
	}
 
 	private boolean hasFunction( String name )
 	{
 		if ( env.getFunction( name ) != null )
 			return true;
 		return false;
 	}
 
	private void callFunction( String name, Stack args, HashMap params )
	{
		ScriptVariables   vars = script.getScriptVariables();
		ScriptEnvironment env  = script.getScriptEnvironment();
		Block code = getBlock( name );
		
//		synchronized (vars)
//		{
			vars.pushClosureLevel( (SleepClosure) env.getFunction( name ) );
			vars.pushLocalLevel();

			Variable localLevel = vars.getLocalVariables();

			vars.setScalarLevel("$0", SleepUtils.getScalar(name), localLevel);

			int n = 1;
			while (!args.isEmpty())
			{
				vars.setScalarLevel("$"+n, (Scalar)args.pop(), localLevel);
				n++;
			}

			vars.setScalarLevel("@_", SleepUtils.getArrayScalar(new ArgumentArray(n, localLevel)), localLevel);
			vars.setScalarLevel( "%_", SleepUtils.getHashScalar( new Parameters(params) ),localLevel );
			
			Scalar temp = code.evaluate(env);

			env.clearReturn();

			vars.popLocalLevel();
			vars.popClosureLevel();
//		}
		//params.clear();
	}

	private Block getBlock( String name )
	{
		Block result = null;
		if ( blocks.containsKey(name) )
		{
			result = (Block) blocks.get(name);
		}
		else
		{
			Function function = env.getFunction( name );
			try
			{
				result = (Block) code.get( function );
				blocks.put( name, result );
			}
			catch ( Exception e ) { e.printStackTrace(); }
		}
		return result;
	}

	private Field getCodeField()
	{
		Field code = null;
		try
		{
			code = SleepClosure.CLOSURE_CLASS.getDeclaredField("code");
			code.setAccessible(true);
		}
		catch ( Exception e ) { e.printStackTrace(); }
		return code;
	}
	

	protected void onConnect() 
	{
		if ( !hasFunction("&onConnect") ) return;
		if ( debug ) log("ScriptedBot.onConnect()");
		Stack args = new Stack();
		HashMap params = new HashMap();		
		callFunction("&onConnect", args, params );
	}

	protected void onDisconnect() 
	{
		if ( !hasFunction("&onDisconnect") ) return;
		if ( debug ) log("ScriptedBot.onDisconnect()");
		Stack args = new Stack();
		HashMap params = new HashMap();
		callFunction("&onDisconnect", args, params );
	}

	protected void onServerResponse(int code, String response) 
	{
		if ( !hasFunction("&onServerResponse") ) return;
		if ( debug ) log("ScriptedBot.onServerResponse( "+code+", "+response+" )");
		
		Stack args = new Stack();
		HashMap params = new HashMap();
		args.push( SleepUtils.getScalar( response ) );
		params.put( "code", args.peek() );
		args.push( SleepUtils.getScalar( code ) );
		params.put( "response", args.peek() );

		callFunction("&onServerResponse", args, params );
	}

	protected void onUserList(String channel, User[] users) 
	{
		if ( !hasFunction("&onUserList") ) return;
		if ( debug ) log("ScriptedBot.onUserList( "+channel+", "+users+" )");
		
		Stack args = new Stack();
		HashMap params = new HashMap();
		Scalar userArray = SleepUtils.getArrayScalar();
		//for ( int i=users.length-1; i>=0; i-- )
		for ( int i=0; i<users.length; i++ )
		{
			userArray.getArray().push( SleepUtils.getScalar( users[i] ) );
		}
		
		args.push( userArray );
		params.put( "users", args.peek() );
		args.push( SleepUtils.getScalar( channel ) );
		params.put( "channel", args.peek() );

		callFunction("&onUserList", args, params );
	}

	protected void onMessage(String channel, String sender, String login, String hostname, String message) 
	{
		if ( !hasFunction("&onMessage") ) return;
		if ( debug ) log("ScriptedBot.onMessage( "+channel+", "+sender+", "+login+", "+hostname+", "+message+" )");
		
		Stack args = new Stack();
		HashMap params = new HashMap();
		args.push( SleepUtils.getScalar( message ) );
		params.put( "message", args.peek() );
		args.push( SleepUtils.getScalar( hostname ) );
		params.put( "hostname", args.peek() );
		args.push( SleepUtils.getScalar( login ) );
		params.put( "login", args.peek() );
		args.push( SleepUtils.getScalar( sender ) );
		params.put( "sender", args.peek() );
		args.push( SleepUtils.getScalar( channel ) );
		params.put( "channel", args.peek() );

		callFunction("&onMessage", args, params );
	}

	protected void onPrivateMessage(String sender, String login, String hostname, String message) 
	{
		if ( !hasFunction("&onPrivateMessage") ) return;
		if ( debug ) log("ScriptedBot.onPrivateMessage( "+sender+", "+login+", "+hostname+", "+message+" )");
	
		Stack args = new Stack();
		HashMap params = new HashMap();
		args.push( SleepUtils.getScalar( message ) );
		params.put( "message", args.peek() );
		args.push( SleepUtils.getScalar( hostname ) );
		params.put( "hostname", args.peek() );
		args.push( SleepUtils.getScalar( login ) );
		params.put( "login", args.peek() );
		args.push( SleepUtils.getScalar( sender ) );
		params.put( "sender", args.peek() );

		callFunction("&onPrivateMessage", args, params );
	}

	protected void onAction(String sender, String login, String hostname, String target, String action) 
	{
		if ( !hasFunction("&onAction") ) return;
		if ( debug ) log("ScriptedBot.onAction( "+sender+", "+login+", "+hostname+", "+target+", "+action+" )");
	
		Stack args = new Stack();
		HashMap params = new HashMap();
		args.push( SleepUtils.getScalar( action ) );
		params.put( "action", args.peek() );
		args.push( SleepUtils.getScalar( target ) );
		params.put( "target", args.peek() );
		args.push( SleepUtils.getScalar( hostname ) );
		params.put( "hostname", args.peek() );
		args.push( SleepUtils.getScalar( login ) );
		params.put( "login", args.peek() );
		args.push( SleepUtils.getScalar( sender ) );
		params.put( "sender", args.peek() );

		callFunction("&onAction", args, params );
	}

	protected void onNotice(String sourceNick, String sourceLogin, String sourceHostname, String target, String notice) 
	{
		if ( !hasFunction("&onNotice") ) return;
		if ( debug ) log("ScriptedBot.onNotice( "+sourceNick+", "+sourceLogin+", "+sourceHostname+", "+target+", "+notice+" )");
	
		Stack args = new Stack();
		HashMap params = new HashMap();
		args.push( SleepUtils.getScalar( notice ) );
		params.put( "notice", args.peek() );
		args.push( SleepUtils.getScalar( target ) );
		params.put( "target", args.peek() );
		args.push( SleepUtils.getScalar( sourceHostname ) );
		params.put( "sourceHostname", args.peek() );
		args.push( SleepUtils.getScalar( sourceLogin ) );
		params.put( "sourceLogin", args.peek() );
		args.push( SleepUtils.getScalar( sourceNick ) );
		params.put( "sourceNick", args.peek() );

		callFunction("&onNotice", args, params );
	}

	protected void onJoin(String channel, String sender, String login, String hostname) 
	{
		if ( !hasFunction("&onJoin") ) return;
		if ( debug ) log("ScriptedBot.onJoin( "+channel+", "+sender+", "+login+", "+hostname+" )");

		Stack args = new Stack();
		HashMap params = new HashMap();
		args.push( SleepUtils.getScalar( hostname ) );
		params.put( "hostname", args.peek() );
		args.push( SleepUtils.getScalar( login ) );
		params.put( "login", args.peek() );
		args.push( SleepUtils.getScalar( sender ) );
		params.put( "sender", args.peek() );
		args.push( SleepUtils.getScalar( channel ) );
		params.put( "channel", args.peek() );
		
		callFunction("&onJoin", args, params );
	}

	protected void onPart(String channel, String sender, String login, String hostname) 
	{
		if ( !hasFunction("&onPart") ) return;
		if ( debug ) log("ScriptedBot.onPart( "+channel+", "+sender+", "+login+", "+hostname+" )");

		Stack args = new Stack();
		HashMap params = new HashMap();
		args.push( SleepUtils.getScalar( hostname ) );
		params.put( "hostname", args.peek() );
		args.push( SleepUtils.getScalar( login ) );
		params.put( "login", args.peek() );
		args.push( SleepUtils.getScalar( sender ) );
		params.put( "sender", args.peek() );
		args.push( SleepUtils.getScalar( channel ) );
		params.put( "channel", args.peek() );
		
		callFunction("&onPart", args, params );
	}

	protected void onNickChange(String oldNick, String login, String hostname, String newNick) 
	{
		if ( !hasFunction("&onNickChange") ) return;
		if ( debug ) log("ScriptedBot.onNickChange( "+oldNick+", "+login+", "+hostname+", "+newNick+" )");

		Stack args = new Stack();
		HashMap params = new HashMap();
		args.push( SleepUtils.getScalar( newNick ) );
		params.put( "newNick", args.peek() );
		args.push( SleepUtils.getScalar( hostname ) );
		params.put( "hostname", args.peek() );
		args.push( SleepUtils.getScalar( login ) );
		params.put( "login", args.peek() );
		args.push( SleepUtils.getScalar( oldNick ) );
		params.put( "oldNick", args.peek() );
		
		callFunction("&onNickChange", args, params );
	}

	protected void onKick(String channel, String kickerNick, String kickerLogin, String kickerHostname, String recipientNick, String reason) 
	{
		if ( !hasFunction("&onKick") ) return;
		if ( debug ) log("ScriptedBot.onKick( "+channel+", "+kickerNick+", "+kickerLogin+", "+kickerHostname+", "+recipientNick+", "+reason+" )");

		Stack args = new Stack();
		HashMap params = new HashMap();
		args.push( SleepUtils.getScalar( reason ) );
		params.put( "reason", args.peek() );
		args.push( SleepUtils.getScalar( recipientNick ) );
		params.put( "recipientNick", args.peek() );
		args.push( SleepUtils.getScalar( kickerHostname ) );
		params.put( "kickerHostname", args.peek() );
		args.push( SleepUtils.getScalar( kickerLogin ) );
		params.put( "kickerLogin", args.peek() );
		args.push( SleepUtils.getScalar( kickerNick ) );
		params.put( "kickerNick", args.peek() );
		args.push( SleepUtils.getScalar( channel ) );
		params.put( "channel", args.peek() );
		
		callFunction("&onKick", args, params );
	}

	protected void onQuit(String sourceNick, String sourceLogin, String sourceHostname, String reason) 
	{
		if ( !hasFunction("&onQuit") ) return;
		if ( debug ) log("ScriptedBot.onQuit( "+sourceNick+", "+sourceLogin+", "+sourceHostname+", "+reason+" )");

		Stack args = new Stack();
		HashMap params = new HashMap();
		args.push( SleepUtils.getScalar( reason ) );
		params.put( "reason", args.peek() );
		args.push( SleepUtils.getScalar( sourceHostname ) );
		params.put( "sourceHostname", args.peek() );
		args.push( SleepUtils.getScalar( sourceLogin ) );
		params.put( "sourceLogin", args.peek() );
		args.push( SleepUtils.getScalar( sourceNick ) );
		params.put( "sourceNick", args.peek() );
		
		callFunction("&onQuit", args, params );
	}
	
/*  DEPRECATED, not implemented for that reason
	protected void onTopic(String channel, String topic) 
	{
	}
*/
	protected void onTopic(String channel, String topic, String setBy, long date, boolean changed) 
	{
		if ( !hasFunction("&onTopic") ) return;
		if ( debug ) log("ScriptedBot.onTopic( "+channel+", "+topic+", "+setBy+", "+date+", "+changed+" )");

		Stack args = new Stack();
		HashMap params = new HashMap();
		args.push( SleepUtils.getScalar( changed ) );
		params.put( "changed", args.peek() );
		args.push( SleepUtils.getScalar( date ) );
		params.put( "date", args.peek() );
		args.push( SleepUtils.getScalar( setBy ) );
		params.put( "setBy", args.peek() );
		args.push( SleepUtils.getScalar( topic ) );
		params.put( "topic", args.peek() );
		args.push( SleepUtils.getScalar( channel ) );
		params.put( "channel", args.peek() );
		
		callFunction("&onTopic", args, params );
	}

	protected void onChannelInfo(String channel, int userCount, String topic) 
	{
		if ( !hasFunction("&onChannelInfo") ) return;
		if ( debug ) log("ScriptedBot.onChannelInfo( "+channel+", "+userCount+", "+topic+" )");

		Stack args = new Stack();
		HashMap params = new HashMap();
		args.push( SleepUtils.getScalar( topic ) );
		params.put( "topic", args.peek() );
		args.push( SleepUtils.getScalar( userCount ) );
		params.put( "userCount", args.peek() );
		args.push( SleepUtils.getScalar( channel ) );
		params.put( "channel", args.peek() );
		
		callFunction("&onChannelInfo", args, params );
	}

	protected void onMode(String channel, String sourceNick, String sourceLogin, String sourceHostname, String mode) 
	{
		if ( !hasFunction("&onMode") ) return;
		if ( debug ) log("ScriptedBot.onMode( "+channel+", "+sourceNick+", "+sourceLogin+", "+sourceHostname+", "+mode+" )");

		Stack args = new Stack();
		HashMap params = new HashMap();
		args.push( SleepUtils.getScalar( mode ) );
		params.put( "mode", args.peek() );
		args.push( SleepUtils.getScalar( sourceHostname ) );
		params.put( "sourceHostname", args.peek() );
		args.push( SleepUtils.getScalar( sourceLogin ) );
		params.put( "sourceLogin", args.peek() );
		args.push( SleepUtils.getScalar( sourceNick ) );
		params.put( "sourceNick", args.peek() );
		args.push( SleepUtils.getScalar( channel ) );
		params.put( "channel", args.peek() );
		
		callFunction("&onMode", args, params );
	}

	protected void onUserMode(String targetNick, String sourceNick, String sourceLogin, String sourceHostname, String mode) 
	{
		if ( !hasFunction("&onUserMode") ) return;
		if ( debug ) log("ScriptedBot.onUserMode( "+targetNick+", "+sourceNick+", "+sourceLogin+", "+sourceHostname+", "+mode+" )");

		Stack args = new Stack();
		HashMap params = new HashMap();
		args.push( SleepUtils.getScalar( mode ) );
		params.put( "mode", args.peek() );
		args.push( SleepUtils.getScalar( sourceHostname ) );
		params.put( "sourceHostname", args.peek() );
		args.push( SleepUtils.getScalar( sourceLogin ) );
		params.put( "sourceLogin", args.peek() );
		args.push( SleepUtils.getScalar( sourceNick ) );
		params.put( "sourceNick", args.peek() );
		args.push( SleepUtils.getScalar( targetNick ) );
		params.put( "targetNick", args.peek() );
		
		callFunction("&onUserMode", args, params );
	}

	protected void onOp(String channel, String sourceNick, String sourceLogin, String sourceHostname, String recipient) 
	{
		if ( !hasFunction("&onOp") ) return;
		if ( debug ) log("ScriptedBot.onOp( "+channel+", "+sourceNick+", "+sourceLogin+", "+sourceHostname+", "+recipient+" )");

		Stack args = new Stack();
		HashMap params = new HashMap();
		args.push( SleepUtils.getScalar( recipient ) );
		params.put( "recipient", args.peek() );
		args.push( SleepUtils.getScalar( sourceHostname ) );
		params.put( "sourceHostname", args.peek() );
		args.push( SleepUtils.getScalar( sourceLogin ) );
		params.put( "sourceLogin", args.peek() );
		args.push( SleepUtils.getScalar( sourceNick ) );
		params.put( "sourceNick", args.peek() );
		args.push( SleepUtils.getScalar( channel ) );
		params.put( "channel", args.peek() );
		
		callFunction("&onOp", args, params );
	}

	protected void onDeop(String channel, String sourceNick, String sourceLogin, String sourceHostname, String recipient) 
	{
		if ( !hasFunction("&onDeop") ) return;
		if ( debug ) log("ScriptedBot.onDeop( "+channel+", "+sourceNick+", "+sourceLogin+", "+sourceHostname+", "+recipient+" )");

		Stack args = new Stack();
		HashMap params = new HashMap();
		args.push( SleepUtils.getScalar( recipient ) );
		params.put( "recipient", args.peek() );
		args.push( SleepUtils.getScalar( sourceHostname ) );
		params.put( "sourceHostname", args.peek() );
		args.push( SleepUtils.getScalar( sourceLogin ) );
		params.put( "sourceLogin", args.peek() );
		args.push( SleepUtils.getScalar( sourceNick ) );
		params.put( "sourceNick", args.peek() );
		args.push( SleepUtils.getScalar( channel ) );
		params.put( "channel", args.peek() );
		
		callFunction("&onDeop", args, params );
	}

	protected void onVoice(String channel, String sourceNick, String sourceLogin, String sourceHostname, String recipient) 
	{
		if ( !hasFunction("&onVoice") ) return;
		if ( debug ) log("ScriptedBot.onVoice( "+channel+", "+sourceNick+", "+sourceLogin+", "+sourceHostname+", "+recipient+" )");

		Stack args = new Stack();
		HashMap params = new HashMap();
		args.push( SleepUtils.getScalar( recipient ) );
		params.put( "recipient", args.peek() );
		args.push( SleepUtils.getScalar( sourceHostname ) );
		params.put( "sourceHostname", args.peek() );
		args.push( SleepUtils.getScalar( sourceLogin ) );
		params.put( "sourceLogin", args.peek() );
		args.push( SleepUtils.getScalar( sourceNick ) );
		params.put( "sourceNick", args.peek() );
		args.push( SleepUtils.getScalar( channel ) );
		params.put( "channel", args.peek() );
		
		callFunction("&onVoice", args, params );
	}

	protected void onDeVoice(String channel, String sourceNick, String sourceLogin, String sourceHostname, String recipient) 
	{
		if ( !hasFunction("&onDeVoice") ) return;
		if ( debug ) log("ScriptedBot.onDeVoice( "+channel+", "+sourceNick+", "+sourceLogin+", "+sourceHostname+", "+recipient+" )");

		Stack args = new Stack();
		HashMap params = new HashMap();
		args.push( SleepUtils.getScalar( recipient ) );
		params.put( "recipient", args.peek() );
		args.push( SleepUtils.getScalar( sourceHostname ) );
		params.put( "sourceHostname", args.peek() );
		args.push( SleepUtils.getScalar( sourceLogin ) );
		params.put( "sourceLogin", args.peek() );
		args.push( SleepUtils.getScalar( sourceNick ) );
		params.put( "sourceNick", args.peek() );
		args.push( SleepUtils.getScalar( channel ) );
		params.put( "channel", args.peek() );
		
		callFunction("&onDeVoice", args, params );
	}

	protected void onSetChannelKey(String channel, String sourceNick, String sourceLogin, String sourceHostname, String key) 
	{
		if ( !hasFunction("&onSetChannelKey") ) return;
		if ( debug ) log("ScriptedBot.onSetChannelKey( "+channel+", "+sourceNick+", "+sourceLogin+", "+sourceHostname+", "+key+" )");

		Stack args = new Stack();
		HashMap params = new HashMap();
		args.push( SleepUtils.getScalar( key ) );
		params.put( "key", args.peek() );
		args.push( SleepUtils.getScalar( sourceHostname ) );
		params.put( "sourceHostname", args.peek() );
		args.push( SleepUtils.getScalar( sourceLogin ) );
		params.put( "sourceLogin", args.peek() );
		args.push( SleepUtils.getScalar( sourceNick ) );
		params.put( "sourceNick", args.peek() );
		args.push( SleepUtils.getScalar( channel ) );
		params.put( "channel", args.peek() );
		
		callFunction("&onSetChannelKey", args, params );
	}

	protected void onRemoveChannelKey(String channel, String sourceNick, String sourceLogin, String sourceHostname, String key) 
	{
		if ( !hasFunction("&onRemoveChannelKey") ) return;
		if ( debug ) log("ScriptedBot.onRemoveChannelKey( "+channel+", "+sourceNick+", "+sourceLogin+", "+sourceHostname+", "+key+" )");

		Stack args = new Stack();
		HashMap params = new HashMap();
		args.push( SleepUtils.getScalar( key ) );
		params.put( "key", args.peek() );
		args.push( SleepUtils.getScalar( sourceHostname ) );
		params.put( "sourceHostname", args.peek() );
		args.push( SleepUtils.getScalar( sourceLogin ) );
		params.put( "sourceLogin", args.peek() );
		args.push( SleepUtils.getScalar( sourceNick ) );
		params.put( "sourceNick", args.peek() );
		args.push( SleepUtils.getScalar( channel ) );
		params.put( "channel", args.peek() );
		
		callFunction("&onRemoveChannelKey", args, params );
	}

	protected void onSetChannelLimit(String channel, String sourceNick, String sourceLogin, String sourceHostname, int limit) 
	{
		if ( !hasFunction("&onSetChannelLimit") ) return;
		if ( debug ) log("ScriptedBot.onSetChannelLimit( "+channel+", "+sourceNick+", "+sourceLogin+", "+sourceHostname+", "+limit+" )");

		Stack args = new Stack();
		HashMap params = new HashMap();
		args.push( SleepUtils.getScalar( limit ) );
		params.put( "limit", args.peek() );
		args.push( SleepUtils.getScalar( sourceHostname ) );
		params.put( "sourceHostname", args.peek() );
		args.push( SleepUtils.getScalar( sourceLogin ) );
		params.put( "sourceLogin", args.peek() );
		args.push( SleepUtils.getScalar( sourceNick ) );
		params.put( "sourceNick", args.peek() );
		args.push( SleepUtils.getScalar( channel ) );
		params.put( "channel", args.peek() );
		
		callFunction("&onSetChannelLimit", args, params );
	}

	protected void onRemoveChannelLimit(String channel, String sourceNick, String sourceLogin, String sourceHostname) 
	{
		if ( !hasFunction("&onRemoveChannelLimit") ) return;
		if ( debug ) log("ScriptedBot.onRemoveChannelLimit( "+channel+", "+sourceNick+", "+sourceLogin+", "+sourceHostname+" )");

		Stack args = new Stack();
		HashMap params = new HashMap();
		args.push( SleepUtils.getScalar( sourceHostname ) );
		params.put( "sourceHostname", args.peek() );
		args.push( SleepUtils.getScalar( sourceLogin ) );
		params.put( "sourceLogin", args.peek() );
		args.push( SleepUtils.getScalar( sourceNick ) );
		params.put( "sourceNick", args.peek() );
		args.push( SleepUtils.getScalar( channel ) );
		params.put( "channel", args.peek() );
		
		callFunction("&onRemoveChannelLimit", args, params );
	}

	protected void onSetChannelBan(String channel, String sourceNick, String sourceLogin, String sourceHostname, String hostmask) 
	{
		if ( !hasFunction("&onSetChannelBan") ) return;
		if ( debug ) log("ScriptedBot.onSetChannelBan( "+channel+", "+sourceNick+", "+sourceLogin+", "+sourceHostname+", "+hostmask+" )");

		Stack args = new Stack();
		HashMap params = new HashMap();
		args.push( SleepUtils.getScalar( hostmask ) );
		params.put( "hostmask", args.peek() );
		args.push( SleepUtils.getScalar( sourceHostname ) );
		params.put( "sourceHostname", args.peek() );
		args.push( SleepUtils.getScalar( sourceLogin ) );
		params.put( "sourceLogin", args.peek() );
		args.push( SleepUtils.getScalar( sourceNick ) );
		params.put( "sourceNick", args.peek() );
		args.push( SleepUtils.getScalar( channel ) );
		params.put( "channel", args.peek() );
		
		callFunction("&onSetChannelBan", args, params );
	}

	protected void onRemoveChannelBan(String channel, String sourceNick, String sourceLogin, String sourceHostname, String hostmask) 
	{
		if ( !hasFunction("&onRemoveChannelBan") ) return;
		if ( debug ) log("ScriptedBot.onRemoveChannelBan( "+channel+", "+sourceNick+", "+sourceLogin+", "+sourceHostname+", "+hostmask+" )");

		Stack args = new Stack();
		HashMap params = new HashMap();
		args.push( SleepUtils.getScalar( hostmask ) );
		params.put( "hostmask", args.peek() );
		args.push( SleepUtils.getScalar( sourceHostname ) );
		params.put( "sourceHostname", args.peek() );
		args.push( SleepUtils.getScalar( sourceLogin ) );
		params.put( "sourceLogin", args.peek() );
		args.push( SleepUtils.getScalar( sourceNick ) );
		params.put( "sourceNick", args.peek() );
		args.push( SleepUtils.getScalar( channel ) );
		params.put( "channel", args.peek() );
		
		callFunction("&onRemoveChannelBan", args, params );
	}

	protected void onSetTopicProtection(String channel, String sourceNick, String sourceLogin, String sourceHostname) 
	{
		if ( !hasFunction("&onSetTopicProtection") ) return;
		if ( debug ) log("ScriptedBot.onSetTopicProtection( "+channel+", "+sourceNick+", "+sourceLogin+", "+sourceHostname+" )");

		Stack args = new Stack();
		HashMap params = new HashMap();
		args.push( SleepUtils.getScalar( sourceHostname ) );
		params.put( "sourceHostname", args.peek() );
		args.push( SleepUtils.getScalar( sourceLogin ) );
		params.put( "sourceLogin", args.peek() );
		args.push( SleepUtils.getScalar( sourceNick ) );
		params.put( "sourceNick", args.peek() );
		args.push( SleepUtils.getScalar( channel ) );
		params.put( "channel", args.peek() );
		
		callFunction("&onSetTopicProtection", args, params );
	}

	protected void onRemoveTopicProtection(String channel, String sourceNick, String sourceLogin, String sourceHostname) 
	{
		if ( !hasFunction("&onRemoveTopicProtection") ) return;
		if ( debug ) log("ScriptedBot.onRemoveTopicProtection( "+channel+", "+sourceNick+", "+sourceLogin+", "+sourceHostname+" )");

		Stack args = new Stack();
		HashMap params = new HashMap();
		args.push( SleepUtils.getScalar( sourceHostname ) );
		params.put( "sourceHostname", args.peek() );
		args.push( SleepUtils.getScalar( sourceLogin ) );
		params.put( "sourceLogin", args.peek() );
		args.push( SleepUtils.getScalar( sourceNick ) );
		params.put( "sourceNick", args.peek() );
		args.push( SleepUtils.getScalar( channel ) );
		params.put( "channel", args.peek() );
		
		callFunction("&onRemoveTopicProtection", args, params );
	}

	protected void onSetNoExternalMessages(String channel, String sourceNick, String sourceLogin, String sourceHostname) 
	{
		if ( !hasFunction("&onSetNoExternalMessages") ) return;
		if ( debug ) log("ScriptedBot.onSetNoExternalMessages( "+channel+", "+sourceNick+", "+sourceLogin+", "+sourceHostname+" )");

		Stack args = new Stack();
		HashMap params = new HashMap();
		args.push( SleepUtils.getScalar( sourceHostname ) );
		params.put( "sourceHostname", args.peek() );
		args.push( SleepUtils.getScalar( sourceLogin ) );
		params.put( "sourceLogin", args.peek() );
		args.push( SleepUtils.getScalar( sourceNick ) );
		params.put( "sourceNick", args.peek() );
		args.push( SleepUtils.getScalar( channel ) );
		params.put( "channel", args.peek() );
		
		callFunction("&onSetNoExternalMessages", args, params );
	}

	protected void onRemoveNoExternalMessages(String channel, String sourceNick, String sourceLogin, String sourceHostname) 
	{
		if ( !hasFunction("&onRemoveNoExternalMessages") ) return;
		if ( debug ) log("ScriptedBot.onRemoveNoExternalMessages( "+channel+", "+sourceNick+", "+sourceLogin+", "+sourceHostname+" )");

		Stack args = new Stack();
		HashMap params = new HashMap();
		args.push( SleepUtils.getScalar( sourceHostname ) );
		params.put( "sourceHostname", args.peek() );
		args.push( SleepUtils.getScalar( sourceLogin ) );
		params.put( "sourceLogin", args.peek() );
		args.push( SleepUtils.getScalar( sourceNick ) );
		params.put( "sourceNick", args.peek() );
		args.push( SleepUtils.getScalar( channel ) );
		params.put( "channel", args.peek() );
		
		callFunction("&onRemoveNoExternalMessages", args, params );
	}

	protected void onSetInviteOnly(String channel, String sourceNick, String sourceLogin, String sourceHostname) 
	{
		if ( !hasFunction("&onSetInviteOnly") ) return;
		if ( debug ) log("ScriptedBot.onSetInviteOnly( "+channel+", "+sourceNick+", "+sourceLogin+", "+sourceHostname+" )");

		Stack args = new Stack();
		HashMap params = new HashMap();
		args.push( SleepUtils.getScalar( sourceHostname ) );
		params.put( "sourceHostname", args.peek() );
		args.push( SleepUtils.getScalar( sourceLogin ) );
		params.put( "sourceLogin", args.peek() );
		args.push( SleepUtils.getScalar( sourceNick ) );
		params.put( "sourceNick", args.peek() );
		args.push( SleepUtils.getScalar( channel ) );
		params.put( "channel", args.peek() );
		
		callFunction("&onSetInviteOnly", args, params );
	}

	protected void onRemoveInviteOnly(String channel, String sourceNick, String sourceLogin, String sourceHostname) 
	{
		if ( !hasFunction("&onRemoveInviteOnly") ) return;
		if ( debug ) log("ScriptedBot.onRemoveInviteOnly( "+channel+", "+sourceNick+", "+sourceLogin+", "+sourceHostname+" )");

		Stack args = new Stack();
		HashMap params = new HashMap();
		args.push( SleepUtils.getScalar( sourceHostname ) );
		params.put( "sourceHostname", args.peek() );
		args.push( SleepUtils.getScalar( sourceLogin ) );
		params.put( "sourceLogin", args.peek() );
		args.push( SleepUtils.getScalar( sourceNick ) );
		params.put( "sourceNick", args.peek() );
		args.push( SleepUtils.getScalar( channel ) );
		params.put( "channel", args.peek() );
		
		callFunction("&onRemoveInviteOnly", args, params );
	}

	protected void onSetModerated(String channel, String sourceNick, String sourceLogin, String sourceHostname) 
	{
		if ( !hasFunction("&onSetModerated") ) return;
		if ( debug ) log("ScriptedBot.onSetModerated( "+channel+", "+sourceNick+", "+sourceLogin+", "+sourceHostname+" )");

		Stack args = new Stack();
		HashMap params = new HashMap();
		args.push( SleepUtils.getScalar( sourceHostname ) );
		params.put( "sourceHostname", args.peek() );
		args.push( SleepUtils.getScalar( sourceLogin ) );
		params.put( "sourceLogin", args.peek() );
		args.push( SleepUtils.getScalar( sourceNick ) );
		params.put( "sourceNick", args.peek() );
		args.push( SleepUtils.getScalar( channel ) );
		params.put( "channel", args.peek() );
		
		callFunction("&onSetModerated", args, params );
	}

	protected void onRemoveModerated(String channel, String sourceNick, String sourceLogin, String sourceHostname) 
	{
		if ( !hasFunction("&onRemoveModerated") ) return;
		if ( debug ) log("ScriptedBot.onRemoveModerated( "+channel+", "+sourceNick+", "+sourceLogin+", "+sourceHostname+" )");

		Stack args = new Stack();
		HashMap params = new HashMap();
		args.push( SleepUtils.getScalar( sourceHostname ) );
		params.put( "sourceHostname", args.peek() );
		args.push( SleepUtils.getScalar( sourceLogin ) );
		params.put( "sourceLogin", args.peek() );
		args.push( SleepUtils.getScalar( sourceNick ) );
		params.put( "sourceNick", args.peek() );
		args.push( SleepUtils.getScalar( channel ) );
		params.put( "channel", args.peek() );
		
		callFunction("&onRemoveModerated", args, params );
	}

	protected void onSetPrivate(String channel, String sourceNick, String sourceLogin, String sourceHostname) 
	{
		if ( !hasFunction("&onSetPrivate") ) return;
		if ( debug ) log("ScriptedBot.onSetPrivate( "+channel+", "+sourceNick+", "+sourceLogin+", "+sourceHostname+" )");

		Stack args = new Stack();
		HashMap params = new HashMap();
		args.push( SleepUtils.getScalar( sourceHostname ) );
		params.put( "sourceHostname", args.peek() );
		args.push( SleepUtils.getScalar( sourceLogin ) );
		params.put( "sourceLogin", args.peek() );
		args.push( SleepUtils.getScalar( sourceNick ) );
		params.put( "sourceNick", args.peek() );
		args.push( SleepUtils.getScalar( channel ) );
		params.put( "channel", args.peek() );
		
		callFunction("&onSetPrivate", args, params );
	}

	protected void onRemovePrivate(String channel, String sourceNick, String sourceLogin, String sourceHostname) 
	{
		if ( !hasFunction("&onRemovePrivate") ) return;
		if ( debug ) log("ScriptedBot.onRemovePrivate( "+channel+", "+sourceNick+", "+sourceLogin+", "+sourceHostname+" )");

		Stack args = new Stack();
		HashMap params = new HashMap();
		args.push( SleepUtils.getScalar( sourceHostname ) );
		params.put( "sourceHostname", args.peek() );
		args.push( SleepUtils.getScalar( sourceLogin ) );
		params.put( "sourceLogin", args.peek() );
		args.push( SleepUtils.getScalar( sourceNick ) );
		params.put( "sourceNick", args.peek() );
		args.push( SleepUtils.getScalar( channel ) );
		params.put( "channel", args.peek() );
		
		callFunction("&onRemovePrivate", args, params );
	}

	protected void onSetSecret(String channel, String sourceNick, String sourceLogin, String sourceHostname) 
	{
		if ( !hasFunction("&onSetSecret") ) return;
		if ( debug ) log("ScriptedBot.onSetSecret( "+channel+", "+sourceNick+", "+sourceLogin+", "+sourceHostname+" )");

		Stack args = new Stack();
		HashMap params = new HashMap();
		args.push( SleepUtils.getScalar( sourceHostname ) );
		params.put( "sourceHostname", args.peek() );
		args.push( SleepUtils.getScalar( sourceLogin ) );
		params.put( "sourceLogin", args.peek() );
		args.push( SleepUtils.getScalar( sourceNick ) );
		params.put( "sourceNick", args.peek() );
		args.push( SleepUtils.getScalar( channel ) );
		params.put( "channel", args.peek() );
		
		callFunction("&onSetSecret", args, params );
	}

	protected void onRemoveSecret(String channel, String sourceNick, String sourceLogin, String sourceHostname) 
	{
		if ( !hasFunction("&onRemoveSecret") ) return;
		if ( debug ) log("ScriptedBot.onRemoveSecret( "+channel+", "+sourceNick+", "+sourceLogin+", "+sourceHostname+" )");

		Stack args = new Stack();
		HashMap params = new HashMap();
		args.push( SleepUtils.getScalar( sourceHostname ) );
		params.put( "sourceHostname", args.peek() );
		args.push( SleepUtils.getScalar( sourceLogin ) );
		params.put( "sourceLogin", args.peek() );
		args.push( SleepUtils.getScalar( sourceNick ) );
		params.put( "sourceNick", args.peek() );
		args.push( SleepUtils.getScalar( channel ) );
		params.put( "channel", args.peek() );
		
		callFunction("&onRemoveSecret", args, params );
	}

	protected void onInvite(String targetNick, String sourceNick, String sourceLogin, String sourceHostname, String channel)  
	{
		if ( !hasFunction("&onInvite") ) return;
		if ( debug ) log("ScriptedBot.onInvite( "+channel+", "+sourceNick+", "+sourceLogin+", "+sourceHostname+" )");

		Stack args = new Stack();
		HashMap params = new HashMap();
		args.push( SleepUtils.getScalar( channel ) );
		params.put( "channel", args.peek() );
		args.push( SleepUtils.getScalar( sourceHostname ) );
		params.put( "sourceHostname", args.peek() );
		args.push( SleepUtils.getScalar( sourceLogin ) );
		params.put( "sourceLogin", args.peek() );
		args.push( SleepUtils.getScalar( sourceNick ) );
		params.put( "sourceNick", args.peek() );
		args.push( SleepUtils.getScalar( targetNick ) );
		params.put( "targetNick", args.peek() );
		
		callFunction("&onInvite", args, params );
	}

	protected void onVersion(String sourceNick, String sourceLogin, String sourceHostname, String target)
	{
		if ( !hasFunction("&onVersion") )
		{	// this.sendRawLine("NOTICE " + sourceNick + " :\u0001VERSION " + _version + "\u0001");
			super.onVersion(sourceNick, sourceLogin, sourceHostname, target);
		}
		else
		{
			if ( debug ) log("ScriptedBot.onVersion( "+sourceNick+", "+sourceLogin+", "+sourceHostname+", "+target+" )");
			
			Stack args = new Stack();
		HashMap params = new HashMap();
			args.push( SleepUtils.getScalar( target ) );
			params.put( "target", args.peek() );
			args.push( SleepUtils.getScalar( sourceHostname ) );
			params.put( "sourceHostname", args.peek() );
			args.push( SleepUtils.getScalar( sourceLogin ) );
			params.put( "sourceLogin", args.peek() );
			args.push( SleepUtils.getScalar( sourceNick ) );
			params.put( "sourceNick", args.peek() );
		
			callFunction("&onVersion", args, params );
		}
	}

	protected void onPing(String sourceNick, String sourceLogin, String sourceHostname, String target, String pingValue)
	{
		if ( !hasFunction("&onVersion") )
		{	// this.sendRawLine("NOTICE " + sourceNick + " :\u0001PING " + pingValue + "\u0001");
			super.onPing(sourceNick, sourceLogin, sourceHostname, target, pingValue);
		}
		else
		{
			if ( debug ) log("ScriptedBot.onPing( "+sourceNick+", "+sourceLogin+", "+sourceHostname+", "+target+", "+pingValue+" )");
			
			Stack args = new Stack();
		HashMap params = new HashMap();
			args.push( SleepUtils.getScalar( pingValue ) );
			params.put( "pingValue", args.peek() );
			args.push( SleepUtils.getScalar( target ) );
			params.put( "target", args.peek() );
			args.push( SleepUtils.getScalar( sourceHostname ) );
			params.put( "sourceHostname", args.peek() );
			args.push( SleepUtils.getScalar( sourceLogin ) );
			params.put( "sourceLogin", args.peek() );
			args.push( SleepUtils.getScalar( sourceNick ) );
			params.put( "sourceNick", args.peek() );
		
			callFunction("&onPing", args, params );
		}
	}
    
    protected void onServerPing(String response)
    {
		if ( !hasFunction("&onServerPing") )
		{	// this.sendRawLine("PONG " + response);
			super.onServerPing( response );
		}
		else
		{
			if ( debug ) log("ScriptedBot.onServerPing( "+response+" )");
			
			Stack args = new Stack();
		HashMap params = new HashMap();
			args.push( SleepUtils.getScalar( response ) );
			params.put( "response", args.peek() );
		
			callFunction("&onServerPing", args, params );
		}
    }
    
    protected void onTime(String sourceNick, String sourceLogin, String sourceHostname, String target)
    {
		if ( !hasFunction("&onTime") )
		{	// this.sendRawLine("NOTICE " + sourceNick + " :\u0001TIME " + new Date().toString() + "\u0001");
			super.onTime(sourceNick, sourceLogin, sourceHostname, target);
		}
		else
		{
			if ( debug ) log("ScriptedBot.onTime( "+sourceNick+", "+sourceLogin+", "+sourceHostname+", "+target+" )");
			
			Stack args = new Stack();
		HashMap params = new HashMap();
			args.push( SleepUtils.getScalar( target ) );
			params.put( "target", args.peek() );
			args.push( SleepUtils.getScalar( sourceHostname ) );
			params.put( "sourceHostname", args.peek() );
			args.push( SleepUtils.getScalar( sourceLogin ) );
			params.put( "sourceLogin", args.peek() );
			args.push( SleepUtils.getScalar( sourceNick ) );
			params.put( "sourceNick", args.peek() );
		
			callFunction("&onTime", args, params );
		}
    }
    
    protected void onFinger(String sourceNick, String sourceLogin, String sourceHostname, String target)
    {
		if ( !hasFunction("&onFinger") )
		{	// this.sendRawLine("NOTICE " + sourceNick + " :\u0001FINGER " + _finger + "\u0001");
			super.onFinger(sourceNick, sourceLogin, sourceHostname, target);
		}
		else
		{
			if ( debug ) log("ScriptedBot.onFinger( "+sourceNick+", "+sourceLogin+", "+sourceHostname+", "+target+" )");
			
			Stack args = new Stack();
		HashMap params = new HashMap();
			args.push( SleepUtils.getScalar( target ) );
			params.put( "target", args.peek() );
			args.push( SleepUtils.getScalar( sourceHostname ) );
			params.put( "sourceHostname", args.peek() );
			args.push( SleepUtils.getScalar( sourceLogin ) );
			params.put( "sourceLogin", args.peek() );
			args.push( SleepUtils.getScalar( sourceNick ) );
			params.put( "sourceNick", args.peek() );
		
			callFunction("&onFinger", args, params );
		}
    }
    
    protected void onUnknown(String line)
    {
		if ( !hasFunction("&onUnknown") ) return;
		if ( debug ) log("ScriptedBot.onUnknown( "+line+" )");
			
		Stack args = new Stack();
		HashMap params = new HashMap();
		args.push( SleepUtils.getScalar( line ) );
		params.put( "line", args.peek() );

		callFunction("&onUnknown", args, params );
	}
    
/*  DEPRECATED, not implemented for that reason
	protected void onDccSendRequest(String sourceNick, String sourceLogin, String sourceHostname, String filename, long address, int port, int size) 
	{
	}
*/
/*  DEPRECATED, not implemented for that reason
	protected void onDccChatRequest(String sourceNick, String sourceLogin, String sourceHostname, long address, int port) 
	{
	}
*/

	protected void onIncomingFileTransfer(DccFileTransfer transfer) 
	{
		if ( !hasFunction("&onIncomingFileTransfer") ) return;
		if ( debug ) log("ScriptedBot.onIncomingFileTransfer( "+transfer+" )");

		Stack args = new Stack();
		HashMap params = new HashMap();
		args.push( SleepUtils.getScalar( transfer ) );
		params.put( "transfer", args.peek() );
		
		callFunction("&onIncomingFileTransfer", args, params );
	}

	protected void onFileTransferFinished(DccFileTransfer transfer, Exception e) 
	{
		if ( !hasFunction("&onFileTransferFinished") ) return;
		if ( debug ) log("ScriptedBot.onFileTransferFinished( "+transfer+", "+e+" )");

		Stack args = new Stack();
		HashMap params = new HashMap();
		if ( e == null )
			args.push( SleepUtils.getEmptyScalar() );
		else
			args.push( SleepUtils.getScalar( e.toString() ) );
		params.put( "exception", args.peek() );		
		args.push( SleepUtils.getScalar( transfer ) );
		params.put( "transfer", args.peek() );		
		
		callFunction("&onFileTransferFinished", args, params );
	}
	
	protected void onIncomingChatRequest(DccChat chat)
	{
		if ( !hasFunction("&onIncomingChatRequest") ) return;
		if ( debug ) log("ScriptedBot.onIncomingChatRequest( "+chat+" )");
		
		Stack args = new Stack();
		HashMap params = new HashMap();
		args.push( SleepUtils.getScalar( chat ) );
		params.put( "chat", args.peek() );		
		
		callFunction("&onIncomingChatRequest", args, params );
	}

	/* Handle multithreaded DccChats */

	protected synchronized void onChatRequestAccepted(DccChat chat)
	{ // callback funktion for dccSendChatRequest
		if ( !hasFunction("&onChatRequestAccepted") ) return;
		if ( debug ) log("ScriptedBot.onChatRequestAccepted( "+chat+" )");
		
		Stack args = new Stack();
		HashMap params = new HashMap();
		if ( chat != null )
			args.push( SleepUtils.getScalar( chat ) );
		else
			args.push( SleepUtils.getEmptyScalar() );
		params.put( "chat", args.peek() );		
		callFunction("&onChatRequestAccepted", args, params );
	}

	protected synchronized void onChatReadLine(DccChat chat, String line)
	{ // callback funktion for dccSendChatRequest
		if ( !hasFunction("&onChatReadLine") ) return;
		if ( debug ) log("ScriptedBot.onChatReadLine( "+chat+" )");
		
		Stack args = new Stack();
		HashMap params = new HashMap();
		
		args.push( SleepUtils.getScalar( line ) );
		params.put( "line", args.peek() );
		args.push( SleepUtils.getScalar( chat ) );
		params.put( "chat", args.peek() );
		
		callFunction("&onChatReadLine", args, params );
	}

	/*
		make them accessible from scripts
		(package private access)
	*/
	void setNameP( String name ) { setName( name ); }
	void setLoginP( String login ) { setLogin( login ); }
	void setVersionP( String version ) { setVersion( version ); }
	void setFingerP( String finger ) { setFinger( finger ); }
	
/*	public void setVerboseP( int verbose )
	{
		if ( verbose == 1 )
			setVerbose(true);
		if ( verbose == 2 )
			this.verbose = true;
		if ( verbose == 3 )
		{
			this.verbose = true;
			setVerbose(true);
		}
	}

    public void logP(String line) {
        if (verbose) {
            //System.out.println(System.currentTimeMillis() + " " + line);
            System.out.println(line);
        }
    } */
   
	public void setVerboseP( boolean verbose )
	{
		setVerbose(verbose);
		this.verbose = verbose;
	}
	
    public void log(String line) {
		if ( !hasFunction("&logAppend") )
		{
			super.log(line);
		}
		else
		{
			//if ( debug ) log("ScriptedBot.logAppend( "+line+" )");
			if (this.verbose)
			{
				Stack args = new Stack();
				HashMap params = new HashMap();
				args.push( SleepUtils.getScalar( System.currentTimeMillis() + " " + line ) );
				params.put( "line", args.peek() );
				callFunction("&logAppend", args, params );
			}
		}
    }
    
	public String toString() {
		return	"Version{" + _version_ + "}" +
				" Connected{" + isConnected() + "}" +
                " Server{" + getServer() + "}" +
                " Port{" + getPort() + "}" +
                " Password{" + getPassword() + "}";
    }
    
    private Scalar toSleepArray( String[] args )
    {
		/*Scalar value = SleepUtils.getArrayScalar();
		//value.getArray().push( SleepUtils.getScalar(args.length) );
		for ( int i=0; i<args.length; i++ )
		{
			value.getArray().push( SleepUtils.getScalar(args[i]) );
		}
		return value;
		*/ // Better make that read-only ;-)
		Set returnValue = new LinkedHashSet();
		for ( int i=0; i<args.length; i++ )
		{
			//System.out.println(args[i]);
			returnValue.add( args[i] );
		}
		return SleepUtils.getArrayWrapper(returnValue); 
    } 
    
	public static void main( String[] args )
	{
		if ( args.length > 0 )
		{
			//arguments = toSleepArray( args );
				
			try
			{
				if ( args.length > 1 )
					new ScriptedBot( args );
				else 
					new ScriptedBot( args[0] );
			}
			catch ( Exception e )
			{
				System.out.println( e.toString() );
				System.exit(1);
			}
		}
		else
		{
			System.out.println("usage: java -jar sleepybot.jar <scriptfilename> or");
			System.out.println("usage: java -jar sleepybot-all.jar <scriptfilename> or");
			System.out.println("usage: java -cp sleepybot.jar;sleep.jar;pircbot.jar org.ululatus.sleepybot.ScriptedBot <scriptfilename>");
			System.exit(1);
		}
	}
	
	private static class Parameters implements ScalarHash
	{
		private HashMap values;

		Parameters( HashMap params )
		{
			values = new HashMap(params);
		}

		public Scalar getAt(Scalar key)
		{
			Object obj = values.get(key.getValue().toString());
			if (obj != null)
			{
				//return SleepUtils.getScalar( obj ); we already put scalars as values
				return (Scalar) obj;
			}
			return SleepUtils.getEmptyScalar();
		}

		public ScalarArray keys()
		{
			return new CollectionWrapper(values.keySet());
		}

		public void remove(Scalar key)
		{
			// do nothing
		}

		public String toString()
		{
			return "(read-only hash " + values.toString() + ")";
		}
	}

}