
package org.ululatus.sleepybot;

import java.util.*;
import java.io.*;
import java.net.*;

import sleep.interfaces.*;
import sleep.runtime.*;
import sleep.bridges.*;

import org.jibble.pircbot.User;
import org.jibble.pircbot.Colors;

public class ScriptedBotBridge implements Loadable, Function
{
	private ScriptedBot scriptbot;
	
	public ScriptedBotBridge( ScriptedBot bot )
	{
		scriptbot = bot;
	}

	public boolean scriptLoaded(ScriptInstance script)
	{
		Variable vars = script.getScriptEnvironment().getScriptVariables().getGlobalVariables();
		//if (scriptbot.getArguments() != null) // never null
		vars.putScalar("@arguments", scriptbot.getArguments());
		vars.putScalar("$empty", SleepUtils.getEmptyScalar());
		
		/* org.jibble.pircbot.Colors */
		vars.putScalar("$NORMAL", SleepUtils.getScalar(Colors.NORMAL));
		vars.putScalar("$BOLD", SleepUtils.getScalar(Colors.BOLD));
		vars.putScalar("$UNDERLINE", SleepUtils.getScalar(Colors.UNDERLINE));
		vars.putScalar("$REVERSE", SleepUtils.getScalar(Colors.REVERSE));
		vars.putScalar("$WHITE", SleepUtils.getScalar(Colors.WHITE));
		vars.putScalar("$BLACK", SleepUtils.getScalar(Colors.BLACK));
		vars.putScalar("$DARK_BLUE", SleepUtils.getScalar(Colors.DARK_BLUE));
		vars.putScalar("$DARK_GREEN", SleepUtils.getScalar(Colors.DARK_GREEN));
		vars.putScalar("$RED", SleepUtils.getScalar(Colors.RED));
		vars.putScalar("$BROWN", SleepUtils.getScalar(Colors.BROWN));
		vars.putScalar("$PURPLE", SleepUtils.getScalar(Colors.PURPLE));
		vars.putScalar("$OLIVE", SleepUtils.getScalar(Colors.OLIVE));		
		vars.putScalar("$YELLOW", SleepUtils.getScalar(Colors.YELLOW));		
		vars.putScalar("$GREEN", SleepUtils.getScalar(Colors.GREEN));		
		vars.putScalar("$TEAL", SleepUtils.getScalar(Colors.TEAL));		
		vars.putScalar("$CYAN", SleepUtils.getScalar(Colors.CYAN));		
		vars.putScalar("$BLUE", SleepUtils.getScalar(Colors.BLUE));		
		vars.putScalar("$MAGENTA", SleepUtils.getScalar(Colors.MAGENTA));		
		vars.putScalar("$DARK_GRAY", SleepUtils.getScalar(Colors.DARK_GRAY));		
		vars.putScalar("$LIGHT_GRAY", SleepUtils.getScalar(Colors.LIGHT_GRAY));	
		
		Hashtable environment = script.getScriptEnvironment().getEnvironment();
		
		/* org.jibble.pircbot.Colors functions */
		environment.put("&removeColors", this);
		environment.put("&removeFormatting", this);
		environment.put("&removeFormattingAndColors", this);
		
		/* some string functions */
		environment.put("&startsWith", this);
		environment.put("&startsWithIgnoreCase", this);
		environment.put("&endsWith", this);
		environment.put("&endsWithIgnoreCase", this);		

		//environment.put("&dumpBlock", this);	
				
		//environment.put("&log", new Log());
		//environment.put("&log2", new Log());
		
		environment.put("&connectServer", new Connect());
		environment.put("&reconnectServer", new Reconnect());
		environment.put("&disconnectServer", new Disconnect());
		environment.put("&quitServer", new QuitServer());
		
		environment.put("&startIdentServer", new StartIdentServer());
		
		environment.put("&joinChannel", new JoinChannel());
		environment.put("&partChannel", new PartChannel());

		environment.put("&sendRawLine", new SendRawLine());
		environment.put("&sendRawLineViaQueue", new SendRawLineViaQueue());

		environment.put("&sendMessage", new SendMessage());
		environment.put("&sendAction", new SendAction());
		environment.put("&sendNotice", new SendNotice());
		environment.put("&sendCTCPCommand", new SendCTCPCommand());
		
		environment.put("&changeNick", new ChangeNick());
		environment.put("&setMode", new SetMode());
		environment.put("&ban", new Ban());
		environment.put("&unBan", new UnBan());
		environment.put("&op", new Op());
		environment.put("&deOp", new DeOp());
		environment.put("&voice", new Voice());
		environment.put("&deVoice", new DeVoice());
		environment.put("&setTopic", new SetTopic());		
		environment.put("&kick", new Kick());
		
		environment.put("&setVerbose", new SetVerbose());
		environment.put("&setName", new SetName());
		environment.put("&setLogin", new SetLogin());
		environment.put("&setVersion", new SetVersion());
		environment.put("&setFinger", new SetFinger());
		
		environment.put("&getName", new GetName());
		environment.put("&getLogin", new GetLogin());
		environment.put("&getVersion", new GetVersion());
		environment.put("&getFinger", new GetFinger());
		environment.put("&isConnected", new IsConnected());
		
		environment.put("&setMessageDelay", new SetMessageDelay());
		environment.put("&getMessageDelay", new GetMessageDelay());
		
		environment.put("&getMaxLineLength", new GetMaxLineLength());
		environment.put("&getOutgoingQueueSize", new GetOutgoingQueueSize());
		environment.put("&getServer", new GetServer());
		environment.put("&getPort", new GetPort());
		environment.put("&getPassword", new GetPassword());
		environment.put("&getNick", new GetNick());
	
		environment.put("&listChannels", new ListChannels());

		environment.put("&getEncoding", new GetEncoding());
		environment.put("&setEncoding", new SetEncoding());

		environment.put("&getChannels", new GetChannels());
		
		environment.put("&getInetAddress", new GetInetAddress());
		environment.put("&getDccInetAddress", new GetDccInetAddress());
		environment.put("&setDccInetAddress", new SetDccInetAddress());
		
		environment.put("&longToIp", new LongToIp());
		environment.put("&ipToLong", new IpToLong());
		
		environment.put("&dispose", new Dispose());

		environment.put("&setAutoNickChange", new SetAutoNickChange());
		
		//environment.put("&getUsers", new GetUsers()); // MOVED BACK HERE FROM UserBridge
		//environment.put("&dccSendFile", new DccSendFile()); // MOVED BACK HERE FROM DccFileTransferBridgeBridge
		
		return true;
	}
	
	public boolean scriptUnloaded(ScriptInstance script)
	{
		return true;
	}

	public Scalar evaluate(String name, ScriptInstance script, Stack args)
	{
		String line = BridgeUtilities.getString( args, "" );
		if ( name.equals("&removeColors") )
		{
			return SleepUtils.getScalar( Colors.removeColors(line) );
		}
		else if ( name.equals("&removeFormatting") )
		{
			return SleepUtils.getScalar( Colors.removeFormatting(line) );
		}
		else if ( name.equals("&removeFormattingAndColors") )
		{
			return SleepUtils.getScalar( Colors.removeFormattingAndColors(line) );
		}
		else if ( name.equals("&startsWith") )
		{
			String str = BridgeUtilities.getString( args, "" );
			if ( line.startsWith(str) )
				return SleepUtils.getScalar( 1 );
			else return SleepUtils.getEmptyScalar();
		}
		else if ( name.equals("&startsWithIgnoreCase") )
		{
			String str = BridgeUtilities.getString( args, "" );
			if ( line.toLowerCase().startsWith(str.toLowerCase()) )
				return SleepUtils.getScalar( 1 );
			else return SleepUtils.getEmptyScalar();
		}
		else if ( name.equals("&endsWith") )
		{
			String str = BridgeUtilities.getString( args, "" );
			if ( line.endsWith(str) )
				return SleepUtils.getScalar( 1 );
			else return SleepUtils.getEmptyScalar();
		}
		else if ( name.equals("&endsWithIgnoreCase") )
		{
			String str = BridgeUtilities.getString( args, "" );
			if ( line.toLowerCase().endsWith(str.toLowerCase()) )
				return SleepUtils.getScalar( 1 );
			else return SleepUtils.getEmptyScalar();
		}
/*		else if ( name.equals("&dumpBlock") )
		{
			String str = BridgeUtilities.getString( args, "" );
			ScriptedBotBridge.this.scriptbot.dumpBlock( str );
		} */
		return SleepUtils.getScalar(line);
	}
	
/*	private class Log implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			String line = BridgeUtilities.getString( args, "" );
			if ( name.equals("&log") )
				ScriptedBotBridge.this.scriptbot.log( line );
			/* else if ( name.equals("&log2") )
				ScriptedBotBridge.this.scriptbot.logP( line ); +/
			return SleepUtils.getEmptyScalar(); // void
		}
	} */
		
    //public final synchronized void connect(String hostname) throws IOException, IrcException, NickAlreadyInUseException {
    //public final synchronized void connect(String hostname, int port) throws IOException, IrcException, NickAlreadyInUseException {
    //public final synchronized void connect(String hostname, int port, String password) throws IOException, IrcException, NickAlreadyInUseException {
	private class Connect implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			String hostname = BridgeUtilities.getString( args, "" );
			int port = -1;
			String password = "";
			if (!args.isEmpty())
			{
				port = BridgeUtilities.getInt( args );
			}

			if (!args.isEmpty())
			{
				password = BridgeUtilities.getString( args, "" );
			}
			try
			{
				if ( !hostname.equals("") && port > 0 && !password.equals("") )
				{
					ScriptedBotBridge.this.scriptbot.connect( hostname, port, password );
				}
				else if ( !hostname.equals("") && port > 0  )
				{
					ScriptedBotBridge.this.scriptbot.connect( hostname, port );
				}
				else if ( !hostname.equals("") )
				{
					ScriptedBotBridge.this.scriptbot.connect( hostname );
				}
				else
				{
					// it sucks
				}
			}
			catch ( Exception e )
			{
				script.getScriptEnvironment().flagError( e.toString() ); 
			}
			return SleepUtils.getEmptyScalar(); // void
		}
	}
	
	
    //public final synchronized void reconnect() throws IOException, IrcException, NickAlreadyInUseException{
	private class Reconnect implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			try
			{
				ScriptedBotBridge.this.scriptbot.reconnect();
			}
			catch ( Exception e )
			{
				script.getScriptEnvironment().flagError( e.toString() ); 
			}
			return SleepUtils.getEmptyScalar(); // void
		}
	}


    //public final synchronized void disconnect() {
	private class Disconnect implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			ScriptedBotBridge.this.scriptbot.disconnect();
			return SleepUtils.getEmptyScalar(); // void
		}
	}

    //public final void quitServer() {
    //public final void quitServer(String reason) {
	private class QuitServer implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			String reason = "";
			if (!args.isEmpty())
			{
				reason = BridgeUtilities.getString( args, "" );
			}
			ScriptedBotBridge.this.scriptbot.quitServer( reason );
			return SleepUtils.getEmptyScalar(); // void
		}
	}

    //public final void startIdentServer() {
	private class StartIdentServer implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			ScriptedBotBridge.this.scriptbot.startIdentServer();
			return SleepUtils.getEmptyScalar(); // void
		}
	}

    //public final void joinChannel(String channel) {
    //public final void joinChannel(String channel, String key) {
	private class JoinChannel implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			String channel = BridgeUtilities.getString( args, "" );
			String key = "";
			if (!args.isEmpty())
			{
				key = BridgeUtilities.getString( args, "" );
			}
			if ( !channel.equals("") && !key.equals("") )
			{
				ScriptedBotBridge.this.scriptbot.joinChannel( channel, key );
			}
			else if ( !channel.equals("") )
			{
				ScriptedBotBridge.this.scriptbot.joinChannel( channel, key );
			}
			else
			{
				// it sucks
			}
			
			return SleepUtils.getEmptyScalar(); // void
		}
	}
    
    
    //public final void partChannel(String channel) {
    //public final void partChannel(String channel, String reason) {
	private class PartChannel implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			String channel = BridgeUtilities.getString( args, "" );
			String reason = "";
			if (!args.isEmpty())
			{
				reason = BridgeUtilities.getString( args, "" );
			}
			if ( !channel.equals("") && !reason.equals("") )
			{
				ScriptedBotBridge.this.scriptbot.partChannel( channel, reason );
			}
			else if ( !channel.equals("") )
			{
				ScriptedBotBridge.this.scriptbot.partChannel( channel );
			}
			else
			{
				// it sucks
			}
			
			return SleepUtils.getEmptyScalar(); // void
		}
	}

    //public final synchronized void sendRawLine(String line) {
	private class SendRawLine implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			String line = BridgeUtilities.getString( args, "" );
			ScriptedBotBridge.this.scriptbot.sendRawLine( line );
			return SleepUtils.getEmptyScalar(); // void
		}
	}

    //public final synchronized void sendRawLineViaQueue(String line) {
	private class SendRawLineViaQueue implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			String line = BridgeUtilities.getString( args, "" );
			ScriptedBotBridge.this.scriptbot.sendRawLineViaQueue( line );
			return SleepUtils.getEmptyScalar(); // void
		}
	}

    //public final void sendMessage(String target, String message) {
	private class SendMessage implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			String target = BridgeUtilities.getString( args, "" );
			String message = BridgeUtilities.getString( args, "" );
			ScriptedBotBridge.this.scriptbot.sendMessage( target, message );
			return SleepUtils.getEmptyScalar(); // void
		}
	}
	
	//public final void sendAction(String target, String action) {
	private class SendAction implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			String target = BridgeUtilities.getString( args, "" );
			String action = BridgeUtilities.getString( args, "" );
			ScriptedBotBridge.this.scriptbot.sendAction( target, action );
			return SleepUtils.getEmptyScalar(); // void
		}
	}
	
    //public final void sendNotice(String target, String notice) {
	private class SendNotice implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			String target = BridgeUtilities.getString( args, "" );
			String notice = BridgeUtilities.getString( args, "" );
			ScriptedBotBridge.this.scriptbot.sendNotice( target, notice );
			return SleepUtils.getEmptyScalar(); // void
		}
	}
	
	//public final void sendCTCPCommand(String target, String command) {
	private class SendCTCPCommand implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			String target = BridgeUtilities.getString( args, "" );
			String command = BridgeUtilities.getString( args, "" );
			ScriptedBotBridge.this.scriptbot.sendCTCPCommand( target, command );
			return SleepUtils.getEmptyScalar(); // void
		}
	}
    
    //public final void changeNick(String newNick) {
	private class ChangeNick implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			String newNick = BridgeUtilities.getString( args, "" );
			ScriptedBotBridge.this.scriptbot.changeNick( newNick );
			return SleepUtils.getEmptyScalar(); // void
		}
	}
    
    //public final void setMode(String channel, String mode) {
	private class SetMode implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			String channel = BridgeUtilities.getString( args, "" );
			String mode = BridgeUtilities.getString( args, "" );
			ScriptedBotBridge.this.scriptbot.setMode( channel, mode );
			return SleepUtils.getEmptyScalar(); // void
		}
	}
    	
    //public final void sendInvite(String nick, String channel) {
	private class SendInvite implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			String nick = BridgeUtilities.getString( args, "" );
			String channel = BridgeUtilities.getString( args, "" );
			ScriptedBotBridge.this.scriptbot.sendInvite( nick, channel );
			return SleepUtils.getEmptyScalar(); // void
		}
	}
    	
    //public final void ban(String channel, String hostmask) {
	private class Ban implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			String channel = BridgeUtilities.getString( args, "" );
			String hostmask = BridgeUtilities.getString( args, "" );
			ScriptedBotBridge.this.scriptbot.ban( channel, hostmask );
			return SleepUtils.getEmptyScalar(); // void
		}
	}
	
    //public final void unBan(String channel, String hostmask) {
	private class UnBan implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			String channel = BridgeUtilities.getString( args, "" );
			String hostmask = BridgeUtilities.getString( args, "" );
			ScriptedBotBridge.this.scriptbot.unBan( channel, hostmask );
			return SleepUtils.getEmptyScalar(); // void
		}
	}
    
    //public final void op(String channel, String nick) {
	private class Op implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			String channel = BridgeUtilities.getString( args, "" );
			String nick = BridgeUtilities.getString( args, "" );
			ScriptedBotBridge.this.scriptbot.op( channel, nick );
			return SleepUtils.getEmptyScalar(); // void
		}
	}
    
    //public final void deOp(String channel, String nick) {
	private class DeOp implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			String channel = BridgeUtilities.getString( args, "" );
			String nick = BridgeUtilities.getString( args, "" );
			ScriptedBotBridge.this.scriptbot.deOp( channel, nick );
			return SleepUtils.getEmptyScalar(); // void
		}
	} 
	
	//public final void voice(String channel, String nick) {
	private class Voice implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			String channel = BridgeUtilities.getString( args, "" );
			String nick = BridgeUtilities.getString( args, "" );
			ScriptedBotBridge.this.scriptbot.voice( channel, nick );
			return SleepUtils.getEmptyScalar(); // void
		}
	}
	
    //public final void deVoice(String channel, String nick) {
	private class DeVoice implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			String channel = BridgeUtilities.getString( args, "" );
			String nick = BridgeUtilities.getString( args, "" );
			ScriptedBotBridge.this.scriptbot.deVoice( channel, nick );
			return SleepUtils.getEmptyScalar(); // void
		}
	}
	
    //public final void setTopic(String channel, String topic) {
	private class SetTopic implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			String channel = BridgeUtilities.getString( args, "" );
			String topic = BridgeUtilities.getString( args, "" );
			ScriptedBotBridge.this.scriptbot.setMode( channel, topic );
			return SleepUtils.getEmptyScalar(); // void
		}
	}
	
    //public final void kick(String channel, String nick) {
    //public final void kick(String channel, String nick, String reason) {
	private class Kick implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			String channel = BridgeUtilities.getString( args, "" );
			String nick = BridgeUtilities.getString( args, "" );
			String reason = "";
			if (!args.isEmpty())
			{
				reason = BridgeUtilities.getString( args, "" );
			}
			if ( !channel.equals("") && !nick.equals("") && !reason.equals("")  )
			{
				ScriptedBotBridge.this.scriptbot.kick( channel, nick, reason );
			}
			else if ( !channel.equals("") && !nick.equals("") )
			{
				ScriptedBotBridge.this.scriptbot.kick( channel, nick );
			}
			else
			{
				// it sucks
			}
			return SleepUtils.getEmptyScalar(); // void
		}
	}

    //public final void listChannels() {
    //public final void listChannels(String parameters) {
    private class ListChannels implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			String parameters = "";
			if (!args.isEmpty())
			{
				parameters = BridgeUtilities.getString( args, "" );
			}
			if ( !parameters.equals("") )
				ScriptedBotBridge.this.scriptbot.listChannels( parameters );
			else
				ScriptedBotBridge.this.scriptbot.listChannels();
			return SleepUtils.getEmptyScalar(); // void
		}
	}

    //public final void setVerbose(boolean verbose) {
	private class SetVerbose implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			int verbose = BridgeUtilities.getInt( args, 0 );
			ScriptedBotBridge.this.scriptbot.setVerboseP( verbose != 0 ? true: false );
			//ScriptedBotBridge.this.scriptbot.setVerboseP( verbose );
			return SleepUtils.getEmptyScalar(); // void
		}
	}

    //protected final void setName(String name) {
	private class SetName implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			String botname = BridgeUtilities.getString( args, "" );
			ScriptedBotBridge.this.scriptbot.setNameP( botname );
			return SleepUtils.getEmptyScalar(); // void
		}
	}
	
    //protected final void setLogin(String login) {
	private class SetLogin implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			String login = BridgeUtilities.getString( args, "" );
			ScriptedBotBridge.this.scriptbot.setLoginP( login );
			return SleepUtils.getEmptyScalar(); // void
		}
	}
	
    //protected final void setVersion(String version) {
	private class SetVersion implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			String version = BridgeUtilities.getString( args, "" );
			ScriptedBotBridge.this.scriptbot.setVersionP( version );
			return SleepUtils.getEmptyScalar(); // void
		}
	}
    
    //protected final void setFinger(String finger) {
	private class SetFinger implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			String finger = BridgeUtilities.getString( args, "" );
			ScriptedBotBridge.this.scriptbot.setFingerP( finger );
			return SleepUtils.getEmptyScalar(); // void
		}
	}
	
    //public final String getName() {
	private class GetName implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			String botname = ScriptedBotBridge.this.scriptbot.getName();			
			return SleepUtils.getScalar( botname ); 
		}
	}
	
    //public final String getLogin() {
	private class GetLogin implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			String login = ScriptedBotBridge.this.scriptbot.getLogin();			
			return SleepUtils.getScalar( login ); 
		}
	}
	
    //public final String getVersion() {
	private class GetVersion implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			String version = ScriptedBotBridge.this.scriptbot.getVersion();			
			return SleepUtils.getScalar( version ); 
		}
	}
	
    //public final String getFinger() {
	private class GetFinger implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			String finger = ScriptedBotBridge.this.scriptbot.getFinger();			
			return SleepUtils.getScalar( finger ); 
		}
	}
	
    //public final synchronized boolean isConnected() {
	private class IsConnected implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			boolean connected = ScriptedBotBridge.this.scriptbot.isConnected();			
			if ( connected )
				return SleepUtils.getScalar( 1 ); 
			else
				return SleepUtils.getEmptyScalar();
		}
	}
	
    //public final void setMessageDelay(long delay) {
	private class SetMessageDelay implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			long delay = BridgeUtilities.getLong( args, 1000l );
			ScriptedBotBridge.this.scriptbot.setMessageDelay( delay );
			return SleepUtils.getEmptyScalar(); // void
		}
	}
	
    //public final long getMessageDelay() {
	private class GetMessageDelay implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			long delay = ScriptedBotBridge.this.scriptbot.getMessageDelay();			
			return SleepUtils.getScalar( delay ); 
		}
	}
	
    //public final int getMaxLineLength() {
	private class GetMaxLineLength implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			int maxLineLength = ScriptedBotBridge.this.scriptbot.getMaxLineLength();			
			return SleepUtils.getScalar( maxLineLength ); 
		}
	}
	
	//public final int getOutgoingQueueSize() {
	private class GetOutgoingQueueSize implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			int outgoingQueueSize = ScriptedBotBridge.this.scriptbot.getOutgoingQueueSize();			
			return SleepUtils.getScalar( outgoingQueueSize ); 
		}
	}
	
    //public final String getServer() {
	private class GetServer implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			String server = ScriptedBotBridge.this.scriptbot.getServer();			
			return SleepUtils.getScalar( server ); 
		}
	}
	
    //public final int getPort() {
	private class GetPort implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			int port = ScriptedBotBridge.this.scriptbot.getPort();			
			return SleepUtils.getScalar( port ); 
		}
	}
	
    //public final String getPassword() {
	private class GetPassword implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			String password = ScriptedBotBridge.this.scriptbot.getPassword();			
			return SleepUtils.getScalar( password ); 
		}
	}
	
	//public String getNick() {
	private class GetNick implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			String nick = ScriptedBotBridge.this.scriptbot.getNick();			
			return SleepUtils.getScalar( nick ); 
		}
	}
	
    //public String getEncoding() {
	private class GetEncoding implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			String encoding = ScriptedBotBridge.this.scriptbot.getEncoding();
			if ( encoding == null )
				return SleepUtils.getEmptyScalar(); 
			else
				return SleepUtils.getScalar( encoding ); 
		}
	}
	
	//public void setEncoding(String charset) throws UnsupportedEncodingException {
	private class SetEncoding implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			String encoding = BridgeUtilities.getString( args, "" );
			try
			{
				ScriptedBotBridge.this.scriptbot.setEncoding( encoding );
			}
			catch ( UnsupportedEncodingException e )
			{
				script.getScriptEnvironment().flagError( e.toString());
			}
			return SleepUtils.getEmptyScalar(); // void
		}
	}
	
	//public final String[] getChannels() {
	private class GetChannels implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			String[] channels = ScriptedBotBridge.this.scriptbot.getChannels();
			Scalar value = SleepUtils.getArrayScalar();
           
			//for ( int i=0; i<channels.length; i++ )
			for ( int i=channels.length-1; i>=0; i-- )
			{
				value.getArray().push( SleepUtils.getScalar(channels[i]) );
			}
			return value;
		}
	}


    //public InetAddress getInetAddress() {
	private class GetInetAddress implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			InetAddress address = ScriptedBotBridge.this.scriptbot.getInetAddress();
			if ( address != null )
				return SleepUtils.getScalar( address ); 
			return SleepUtils.getEmptyScalar(); 
		}
	}

    //public void setDccInetAddress(InetAddress dccInetAddress) {
	private class SetDccInetAddress implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			Object obj = BridgeUtilities.getObject( args );
			if ( obj != null && obj instanceof InetAddress )
				ScriptedBotBridge.this.scriptbot.setDccInetAddress( (InetAddress) obj );
			else
				ScriptedBotBridge.this.scriptbot.setDccInetAddress( null );
			return SleepUtils.getEmptyScalar(); // void
		}
	}
    
    //public InetAddress getDccInetAddress() {
	private class GetDccInetAddress implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			InetAddress address = ScriptedBotBridge.this.scriptbot.getDccInetAddress();
			if ( address != null )
				return SleepUtils.getScalar( address ); 
			return SleepUtils.getEmptyScalar(); 
		}
	}

    //public int[] longToIp(long address) {
	private class LongToIp implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			long address = BridgeUtilities.getLong( args );
			int[] ip = ScriptedBotBridge.this.scriptbot.longToIp( address );
			Scalar result = SleepUtils.getArrayScalar();           
			//for ( int i=ip.length-1; i>=0; i-- )
			for ( int i=0; i<ip.length; i++ )
			{
				result.getArray().push( SleepUtils.getScalar(ip[i]));
			}
			return result;
		}
	}
	
	//public long ipToLong(byte[] address) {
/*	private class IpToLong implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			ScalarArray ip = BridgeUtilities.getArray(args);
			if ( ip.size() == 4 ) // pircbot supports only ip4 addresses
			{
				byte[] bytes = new byte[4];
				int count = 0;
				Iterator iter = ip.scalarIterator();
				while (iter.hasNext())
				{
					int i = ((Scalar)iter.next()).intValue();
					if ( !(i >= 0 && i < 256) )
					{
						script.getScriptEnvironment().flagError( "ipToLong: values must be in range 0 - 255" );
						return SleepUtils.getEmptyScalar();
					}
					bytes[count++] = new Integer(i).byteValue();
				}
				return SleepUtils.getScalar( ScriptedBotBridge.this.scriptbot.ipToLong( bytes ) );
			}
			else
				script.getScriptEnvironment().flagError( "ipToLong: array must be of length 4" );
			return SleepUtils.getEmptyScalar();
		}
	} */
	
	private class IpToLong implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			Object obj = BridgeUtilities.getObject(args);
			if ( obj != null && obj instanceof byte[] )
			{
				byte[] ip = (byte[]) obj;
				if ( ip.length == 4 ) // pircbot supports only ip4 addresses
				{
					return SleepUtils.getScalar( ScriptedBotBridge.this.scriptbot.ipToLong( ip ) );
				}
				else
					script.getScriptEnvironment().flagError( "ipToLong: array must be of length 4" );
			}
			return SleepUtils.getEmptyScalar();
		}
	}
	
	//public int[] getDccPorts() {
	private class GetDccPorts implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			int[] ports = ScriptedBotBridge.this.scriptbot.getDccPorts();
			Scalar result = SleepUtils.getArrayScalar(); 
			if ( ports != null )
			{
				//Scalar result = SleepUtils.getArrayScalar();           
				//for ( int i=ports.length-1; i>=0; i-- )
				for ( int i=0; i<ports.length; i++ )
				{
					result.getArray().push( SleepUtils.getScalar(ports[i]));
				}
				return result;
			}
			else
				return SleepUtils.getEmptyScalar();
		}
	}

	//public void setDccPorts(int[] ports) {
	private class SetDccPorts implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			ScalarArray pa = BridgeUtilities.getArray(args);
			if ( pa != null && pa.size() > 0 )
			{
				int[] ports = new int[pa.size()];
				int count = 0;
				Iterator iter = pa.scalarIterator();
				while (iter.hasNext())
				{
					ports[count++] = ((Scalar)iter.next()).intValue();
				}
				ScriptedBotBridge.this.scriptbot.setDccPorts( ports );
			}
			else
				ScriptedBotBridge.this.scriptbot.setDccPorts( null );
			return SleepUtils.getEmptyScalar();
		}
	}
	
	// public synchronized void dispose() {
	private class Dispose implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			ScriptedBotBridge.this.scriptbot.dispose();
			return SleepUtils.getEmptyScalar();
		}
	}
	
	private class SetAutoNickChange implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			int value = BridgeUtilities.getInt( args, 0 );
			ScriptedBotBridge.this.scriptbot.setAutoNickChange( (value != 0 ? true : false) );
			return SleepUtils.getEmptyScalar();
		}
	}
	
/*
	PENDING: IMPLEMENTATION
    
    public boolean equals(Object o) {
    public int hashCode() {
    public String toString() {

	MOVED TO UserBridge:
	public final User[] getUsers(String channel) {
	
    MOVED TO DccChatBridge:
	public final DccChat dccSendChatRequest(String nick, int timeout) {
	
    MOVED TO DccFileTransferBridge:
	public final DccFileTransfer dccSendFile(File file, String nick, int timeout) {
      
	DEPRECATED, not implemented for that reason
    protected final void dccReceiveFile(File file, long address, int port, int size) {
    protected final DccChat dccAcceptChatRequest(String sourceNick, long address, int port) {
*/

}
