
package org.ululatus.sleepybot;

import java.util.*;

import sleep.interfaces.*;
import sleep.runtime.*;
import sleep.bridges.*;
import sleep.parser.ParserConfig;

import org.jibble.pircbot.User;

	/* org.jibble.pircbot.User functions */

public class UserBridge implements Loadable, Function
{
	//private static ScriptedBot scriptbot;
	private ScriptedBot scriptbot;

	static { // add Predicate keywords
		ParserConfig.addKeyword("-isuser");
		ParserConfig.addKeyword("-isop");
		ParserConfig.addKeyword("-hasvoice");
	}
	
	public UserBridge(ScriptedBot bot)
	{
		scriptbot = bot;
	}

	public boolean scriptLoaded(ScriptInstance script)
	{
		Hashtable environment = script.getScriptEnvironment().getEnvironment();
		environment.put("&getUsers", new GetUsers()); 
		
		IsUser isuser = new IsUser();
		environment.put("-isuser", isuser);
		environment.put("&isUser", isuser);
		
		IsOp isop = new IsOp();
		environment.put("-isop", isop);
		environment.put("&userIsOp", isop);
		
		HasVoice hasvoice = new HasVoice();
		environment.put("-hasvoice", hasvoice);
		environment.put("&userHasVoice", hasvoice);
		
		environment.put("&userGetNick", this);
		environment.put("&userGetPrefix", this);
		environment.put("&userToString", this);
		environment.put("&userCompareTo", this);
		environment.put("&userEqualsUser", this);
		environment.put("&userEqualsNick", this);
		environment.put("&userHashCode", this);
				
		return true;
	}
	
	public boolean scriptUnloaded(ScriptInstance script)
	{
		return true;
	}


	public Scalar evaluate(String name, ScriptInstance script, Stack args)
	{
		Object obj = BridgeUtilities.getObject( args );
		if ( obj instanceof User )
		{
			User user = (User) obj;
			if ( name.equals("&userGetNick") )
			{
				return SleepUtils.getScalar( user.getNick() );
			}
			else if ( name.equals("&userGetPrefix") )
			{
				return SleepUtils.getScalar( user.getPrefix() );
			}
			else if ( name.equals("&userToString") )
			{
				return SleepUtils.getScalar( user.toString() );
			}
			else if ( name.equals("&userCompareTo") )
			{
				Object other = BridgeUtilities.getObject( args );
				return SleepUtils.getScalar( user.compareTo( other ) );
			}
			else if ( name.equals("&userEqualsUser") )
			{
				Object other = BridgeUtilities.getObject( args );
				if ( user.equals( other ) )
					return SleepUtils.getScalar( 1 );
				else
					return SleepUtils.getEmptyScalar();
			}
			else if ( name.equals("&userEqualsNick") )
			{
				String other = BridgeUtilities.getString( args, "" );
				if ( user.equals( other ) )
					return SleepUtils.getScalar( 1 );
				else
					return SleepUtils.getEmptyScalar();
			}
			else if ( name.equals("&userHashCode") )
			{
				return SleepUtils.getScalar( user.hashCode() );
			}
		}
		return SleepUtils.getEmptyScalar();
	}

	
	//public final User[] getUsers(String channel) {
//	MOVED BACK TO ScriptedBotBridge
	private class GetUsers implements Function
	{
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			String channel = BridgeUtilities.getString( args, "" );
			User[] users = UserBridge.this.scriptbot.getUsers( channel );
			
			Scalar value = SleepUtils.getArrayScalar();
			
			//for ( int i=users.length-1; i>=0; i-- )
			for ( int i=0; i<users.length; i++ )
			{
				value.getArray().push( SleepUtils.getScalar(users[i]) );
			}
			return value;
		}
	}

	private class IsUser implements Predicate, Function
	{
		public boolean decide(String name, ScriptInstance script, Stack args)
		{
			Object a = BridgeUtilities.getObject( args );
			return ( a instanceof User );
		}
		
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			Object obj = BridgeUtilities.getObject( args );
			if ( obj instanceof User )
				return SleepUtils.getScalar(1);
			return SleepUtils.getEmptyScalar();
		}
	}
    
	private class IsOp implements Predicate, Function
	{
		public boolean decide(String name, ScriptInstance script, Stack args)
		{
			Object obj = BridgeUtilities.getObject( args );
			if ( obj instanceof User )
			{
				return ((User) obj).isOp();
			}
			return false;
		}

		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			Object obj = BridgeUtilities.getObject( args );
			if ( obj instanceof User )
			{
				if ( ((User) obj).isOp() )
					return SleepUtils.getScalar(1);
			}	
			return SleepUtils.getEmptyScalar();
		}
	}
    
	private class HasVoice implements Predicate, Function
	{
		public boolean decide(String name, ScriptInstance script, Stack args)
		{
			Object obj = BridgeUtilities.getObject( args );
			if ( obj instanceof User )
			{
				return ((User) obj).hasVoice();
			}
			return false;
		}
		
		public Scalar evaluate(String name, ScriptInstance script, Stack args)
		{
			Object obj = BridgeUtilities.getObject( args );
			if ( obj instanceof User )
			{
				if ( ((User) obj).hasVoice() )
					return SleepUtils.getScalar(1);
			}	
			return SleepUtils.getEmptyScalar();
		}
	}
    
}
